/***********************************************************
Main code and 
Holonomic Drive Vehicle with Stochastic Evolution

William Westrick
Nicole Rodia
************************************************************/

/***********************************************************
* INCLUDES
************************************************************/
#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>
#include <avr/interrupt.h>
#include <string.h>
#include <math.h>

#include "main.h"
#include "uart.h"
#include "mouse_sensor.h"
#include "motor_pwm.h"
#include "evolution.h"

/***********************************************************
* FUNCTION PROTOTYPES
************************************************************/
//Init timers, uart, variables
void init(void); 

/***********************************************************
*  ISR(TIMER0_COMPA_vect)
*		Timer 0: 1 ms interrupt
************************************************************/
ISR(TIMER0_COMPA_vect)
{
	//temporary status for each motor pin
	unsigned char motor_activators;

	//every 100 counts, reset PWM timer
	// this creates a 1khz effective PWM period
	if (!(--PWM_time))
	{
		//reset the counters
		PWM_time=MAX_PWM_TIME;
		motor1_PWM_counter=motor1_PWM;
		motor2_PWM_counter=motor2_PWM;
		motor3_PWM_counter=motor3_PWM;

		//decrement task times if they are not zero
		// this is every 1ms, so task times are in ms
		if (motorTime) {
			motorTime--;
		}
		if (readTime){
			readTime--;
		}
		if (sendTime){
			sendTime--;
		}
		if (evolutionTime){
			evolutionTime--;
		}
	}

	//clear the motor activators	
	motor_activators = 0;
	//check if each motor should be on
	if (motor1_PWM_counter)
	{
		//decrement counter
		motor1_PWM_counter--;
		//set motor on, in the specified direction
		motor_activators |= motor1_DIR;
	}
	//do the same for the other 2 motors
	if (motor2_PWM_counter)
	{
		motor2_PWM_counter--;
		motor_activators |= motor2_DIR;
	}	
	if (motor3_PWM_counter)
	{
		motor3_PWM_counter--;
		motor_activators |= motor3_DIR;
	}

	PORTA = 0;					//prevent shoot-through, etc
	PORTA = motor_activators;	//set pins appropriately

} //ISR(TIMER0_COMPA_vect)

/***********************************************************
* int main(void)
*		main function
*		Coordinates tasks, initialization
************************************************************/
int main(void)
{
	//let all ICs stabilize before attempting communications
	_delay_ms(100);	
	//Initialize everything
	init();	
	//wait to make sure everything is setup
	_delay_ms(100);

	while(1) 
	{	//inifite loop - main program function

		//Turn off motors after 1 second 
		if(!motorTime)
		{
			//see if motors should be turned off
			if (runningMember && !doneRunningMember)
			{
				stopPhenotype();
			}
		}

		//time to run evolution?
		if(!evolutionTime)
		{
			evolutionTime=evolutionTimeMax; //reset counter
			PORTD |= (1<<PD2); //turn off led
			ManageEvolution();	//run evolution
		}

		//Read motion data from optical sensors
		if(!readTime) 
		{
			//reset counter
			readTime = readTimeMax;
			while(Read_Data(0)){} //read while there is data to get
			while(Read_Data(1)){} //read while there is data to get
			ReadSurfaceQuality(); //read the surface quality for monitoring
			Calc_Pos();	//calculate the position
		}
		//Send motion data over uart 
		if(!sendTime) //only send data if there was motion
		{			
			//reset the counter
			sendTime = sendTimeMax;
			Send_Data(); //send the data over UART
		}
	}

} //int main(void)

/***********************************************************
* void Send_Data(void)
* 		Send status data over UART
************************************************************/
void Send_Data(void)
{	

	//Extract the bytes for sending to MATLAB with binary interpretation
/*
	//extract the bytes and right align them
	temp_str[0] = (char)((sum_x[0]&0xff000000)>>24);
	temp_str[1] = (char)((sum_x[0]&0x00ff0000)>>16);
	temp_str[2] = (char)((sum_x[0]&0x0000ff00)>>8);
	temp_str[3] = (char)((sum_x[0]&0x000000ff));
	temp_str[4] = (char)((sum_y[0]&0xff000000)>>24);
	temp_str[5] = (char)((sum_y[0]&0x00ff0000)>>16);
	temp_str[6] = (char)((sum_y[0]&0x0000ff00)>>8);
	temp_str[7] = (char)((sum_y[0]&0x000000ff));
	temp_str[8] = (char)((sum_x[1]&0xff000000)>>24);
	temp_str[9] = (char)((sum_x[1]&0x00ff0000)>>16);
	temp_str[10] = (char)((sum_x[1]&0x0000ff00)>>8);
	temp_str[11] = (char)((sum_x[1]&0x000000ff)); 
	temp_str[12] = (char)((sum_y[1]&0xff000000)>>24);
	temp_str[13] = (char)((sum_y[1]&0x00ff0000)>>16);
	temp_str[14] = (char)((sum_y[1]&0x0000ff00)>>8);
	temp_str[15] = (char)((sum_y[1]&0x000000ff));
*/
	//Send the calculated values
/*	temp_str[0] = (char)((((long)(xval*1024))&0xff000000)>>24);
	temp_str[1] = (char)((((long)(xval*1024))&0x00ff0000)>>16);
	temp_str[2] = (char)((((long)(xval*1024))&0x0000ff00)>>8);
	temp_str[3] = (char)((((long)(xval*1024))&0x000000ff));
	temp_str[4] = (char)((((long)(yval*1024))&0xff000000)>>24);
	temp_str[5] = (char)((((long)(yval*1024))&0x00ff0000)>>16);
	temp_str[6] = (char)((((long)(yval*1024))&0x0000ff00)>>8);
	temp_str[7] = (char)((((long)(yval*1024))&0x000000ff));
	temp_str[24] = (char)((((long)(omega*1024))&0xff000000)>>24);
	temp_str[25] = (char)((((long)(omega*1024))&0x00ff0000)>>16);
	temp_str[26] = (char)((((long)(omega*1024))&0x0000ff00)>>8);
	temp_str[27] = (char)((((long)(omega*1024))&0x000000ff));*/
	
	//Send absolute position and rotation
/*	temp_str[0] = (char)((((long)(absx*1024))&0xff000000)>>24);
	temp_str[1] = (char)((((long)(absx*1024))&0x00ff0000)>>16);
	temp_str[2] = (char)((((long)(absx*1024))&0x0000ff00)>>8);
	temp_str[3] = (char)((((long)(absx*1024))&0x000000ff));
	temp_str[4] = (char)((((long)(absy*1024))&0xff000000)>>24);
	temp_str[5] = (char)((((long)(absy*1024))&0x00ff0000)>>16);
	temp_str[6] = (char)((((long)(absy*1024))&0x0000ff00)>>8);
	temp_str[7] = (char)((((long)(absy*1024))&0x000000ff));
	temp_str[8] = (char)((((long)(absrot*1024))&0xff000000)>>24);
	temp_str[9] = (char)((((long)(absrot*1024))&0x00ff0000)>>16);
	temp_str[10] = (char)((((long)(absrot*1024))&0x0000ff00)>>8);
	temp_str[11] = (char)((((long)(absrot*1024))&0x000000ff));	*/
				
	//Send data to the UART
	//uart_send_str(&temp_str,4);
	
	//send the position information as a string over UART
	//sprintf(temp_str,"x: %+ld\t y: %+ld\t rot: %+ld\t squal1: %d\t squal2: %d\n\r", (long)xval2, (long)yval, (long)omega, squal[0], squal[1]);
	//uart_send_str(temp_str,strlen(temp_str));

	
	//send evolution status over UART for monitoring
	sprintf(temp_str,"e: %d\t m: %d\t 1: %d\t 2: %d\t 3: %d\t x: %+ld\t y: %+ld r: %+ld\t\t\n\r", epoch, curMember,
					 population[curMember].motors.chars[0], population[curMember].motors.chars[1], population[curMember].motors.chars[2],
											(long)xval, (long)yval, (long)omega);
	uart_send_str(temp_str,strlen(temp_str));
	
	//send the evolution statistics over UART
	sprintf(temp_str,"min: %+ld\t max: %+ld\t avg: %+ld\n\r",minfitness,WorstFitness,displayavgfitness);
	uart_send_str(temp_str,strlen(temp_str));
}

/***********************************************************
* void init(void)
* 		Configures the processor
* 		Sets up I/Os and calls UART initialization
*		Sets up timer 0
*		Sets initial variable values
************************************************************/
void init(void)
{
	//set the ports up
	DDRA |= 0xFF; //all outputs
	PORTA = 0x00;
	DDRD |= (1<<PD2); //output for LED 

	//setup timer 0 for 100kHz interrupt for 1khz pwm and ms timer
	TCCR0A = (1<<WGM01); 	//CTC mode
	TCCR0B = (1<<CS01);		//prescale by 8
	OCR0A = 24;				//interrupt every 25 ticks -> 100kHz
	TIMSK0 = (1<<OCIE0A);	//timer 0 compare match

	//init the UART
	uart_init();
	//show that it is working
	uart_send_str("\033[2J\033[HInitialize\n\r",19);
	
	//Set initial timer values
	readTime = readTimeMax;
	sendTime = sendTimeMax;
	evolutionTime = evolutionTimeMax;
	motorTime = 0;

	//initialize the functionality

	initializeMotors();
	initializeEvolution();

	//Enable Interrupts
	sei();

	//configure the mouse sensors
	// do this AFTER enabling interrupts so it can send stuff over SPI	
	initializeMouseSensors();
}
